package cn.iocoder.yudao.framework.mybatis.core.handler;

import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.entity.BaseEntity;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import org.apache.ibatis.reflection.MetaObject;

import java.util.Objects;

/**
 * 通用参数填充实现类
 * 如果没有显式的对通用参数进行赋值，这里会对通用参数进行填充、赋值
 * @author hexiaowu
 */
public class DefaultDBFieldHandler implements MetaObjectHandler {

    @Override
    public void insertFill(MetaObject metaObject) {
        if (Objects.nonNull(metaObject)) {
            Object object = metaObject.getOriginalObject();
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            if (object instanceof BaseDO) {
                BaseDO baseDO = (BaseDO) object;
                // 创建时间为空，则以当前时间为插入时间
                if (Objects.isNull(baseDO.getCreateTime())) {
                    baseDO.setCreateTime(DateUtil.date());
                }
                // 更新时间为空，则以当前时间为更新时间
                if (Objects.isNull(baseDO.getUpdateTime())) {
                    baseDO.setUpdateTime(DateUtil.date());
                }
                // 当前登录用户不为空，创建人为空，则当前登录用户为创建人
                if (Objects.nonNull(userId) && Objects.isNull(baseDO.getCreator())) {
                    baseDO.setCreator(userId.toString());
                }
                // 当前登录用户不为空，更新人为空，则当前登录用户为更新人
                if (Objects.nonNull(userId) && Objects.isNull(baseDO.getUpdater())) {
                    baseDO.setUpdater(userId.toString());
                }
            }
            if (object instanceof BaseEntity) {
                BaseEntity entity = (BaseEntity) object;
                // 创建时间为空，则以当前时间为插入时间
                if (Objects.isNull(entity.getCreatedAt())) {
                    entity.setCreatedAt(DateUtil.date());
                }
                // 更新时间为空，则以当前时间为更新时间
                if (Objects.isNull(entity.getUpdatedAt())) {
                    entity.setUpdatedAt(DateUtil.date());
                }
                // 当前登录用户不为空，创建人为空，则当前登录用户为创建人
                if (Objects.nonNull(userId) && Objects.isNull(entity.getCreatedBy())) {
                    entity.setCreatedBy(userId);
                }
                // 当前登录用户不为空，更新人为空，则当前登录用户为更新人
                if (Objects.nonNull(userId) && Objects.isNull(entity.getUpdatedBy())) {
                    entity.setUpdatedBy(userId);
                }
            }
        }
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        // 更新时间为空，则以当前时间为更新时间
        Object modifyTime = getFieldValByName("updateTime", metaObject);
        if (Objects.isNull(modifyTime)) {
            setFieldValByName("updateTime", DateUtil.date(), metaObject);
        }
        modifyTime = getFieldValByName("updatedAt", metaObject);
        if (Objects.isNull(modifyTime)) {
            setFieldValByName("updatedAt", DateUtil.date(), metaObject);
        }

        // 当前登录用户不为空，更新人为空，则当前登录用户为更新人
        Object modifier = getFieldValByName("updater", metaObject);
        if (Objects.nonNull(userId) && Objects.isNull(modifier)) {
            setFieldValByName("updater", userId.toString(), metaObject);
        }

        modifier = getFieldValByName("updatedBy", metaObject);
        if (Objects.nonNull(userId) && Objects.isNull(modifier)) {
            setFieldValByName("updatedBy", userId, metaObject);
        }
    }
}
