package cn.iocoder.yudao.framework.sms.core.client.dto;

import lombok.Data;

/**
 * SendChamp发送返回结果
 *
 * 该类用于封装SendChamp发送操作的返回结果，包括发送状态码、数据信息、错误信息等
 * 主要用于处理和解析SendChamp API的响应内容
 *
 * @author Jayden
 * @date 2024/11/23
 */
@Data
public class SendChampResult {
    // 响应状态码，用于表示发送操作的结果状态
    private int code;
    // 发送数据信息，包括发送消息的ID、业务ID和总联系人数
    private Data data;
    // 错误信息，用于描述发送操作中可能出现的错误详情
    private String errors;
    // 消息信息，通常用于提供更详细的响应描述
    private String message;
    // 响应状态，以字符串形式表示发送操作的状态
    private String status;

    /**
     * 发送数据信息
     * 该内部类用于详细描述发送操作的相关数据，包括发送消息的唯一标识、业务标识和联系人的总数
     */
    @lombok.Data
    public static class Data {
        // 发送消息的唯一标识符
        private String id;
        // 业务标识符，用于关联发送消息的特定业务
        private String businessId;
        // 总联系人数，表示发送消息所涉及的联系人数量
        private int totalContacts;
    }
}
