package cn.iocoder.yudao.framework.sms.core.client.dto;

import cn.iocoder.yudao.framework.common.util.spring.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.sms.core.enums.SmsFrameworkErrorCodeConstants;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.Data;
import lombok.ToString;

import java.util.Date;
import java.util.Map;

/**
 * 日志dto
 *
 * @author Jayden
 * @date 2024/11/6
 */
@Data
@ToString(callSuper = true)
public class SmsLogDTO {

    /**
     * 自增编号
     */
    private Long id;

    // ========= 渠道相关字段 =========

    /**
     * 短信渠道编号
     * <p>
     */
    private Long channelId;
    /**
     * 短信渠道编码
     * <p>
     */
    private String channelCode;

    // ========= 模板相关字段 =========

    /**
     * 模板编号
     * <p>
     */
    private Long templateId;
    /**
     * 模板编码
     * <p>
     */
    private String templateCode;
    /**
     * 短信类型
     * <p>
     */
    private Integer templateType;
    /**
     *
     */
    private String templateContent;
    /**
     *
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Map<String, Object> templateParams;
    /**
     * 短信 API 的模板编号
     * <p>
     */
    private String apiTemplateId;

    // ========= 手机相关字段 =========

    /**
     * 手机号
     */
    private String mobile;
    /**
     * 用户编号
     */
    private Long userId;
    /**
     * 用户类型
     * <p>
     * 枚举 {@link UserTypeEnum}
     */
    private Integer userType;

    // ========= 发送相关字段 =========

    /**
     * 发送状态
     * <p>
     */
    private Integer sendStatus;
    /**
     * 发送时间
     */
    private Date sendTime;
    /**
     * 发送结果的编码
     * <p>
     * 枚举 {@link SmsFrameworkErrorCodeConstants}
     */
    private Integer sendCode;
    /**
     * 发送结果的提示
     * <p>
     * 一般情况下，使用 {@link SmsFrameworkErrorCodeConstants}
     * 异常情况下，通过格式化 Exception 的提示存储
     */
    private String sendMsg;
    /**
     * 短信 API 发送结果的编码
     * <p>
     * 由于第三方的错误码可能是字符串，所以使用 String 类型
     */
    private String apiSendCode;
    /**
     * 短信 API 发送失败的提示
     */
    private String apiSendMsg;
    /**
     * 短信 API 发送返回的唯一请求 ID
     * <p>
     * 用于和短信 API 进行定位于排错
     */
    private String apiRequestId;
    /**
     * 短信 API 发送返回的序号
     * <p>
     * 用于和短信 API 平台的发送记录关联
     */
    private String apiSerialNo;

    // ========= 接收相关字段 =========

    /**
     * 接收状态
     * <p>
     */
    private Integer receiveStatus;
    /**
     * 接收时间
     */
    private Date receiveTime;
    /**
     * 短信 API 接收结果的编码
     */
    private String apiReceiveCode;
    /**
     * 短信 API 接收结果的提示
     */
    private String apiReceiveMsg;

    /**
     * 重发的短信日志id
     */
    private Long smsLogId;
    /**
     * 重发次数
     */
    private Integer num;

    /**
     * 短信节点表id
     */
    private Long nodeId;

    /**
     * 节点模板序列号
     */
    private Integer nodeTemplateSn;

    /**
     * 发送类型
     */
    private Integer messageType;
}
